@@ -14,7 +14,7 @@
 
 #ifdef AUTH_PERL
 
-static const char rcsid[] = "$Id: auth_perl.c,v 1.24 2002/06/25 20:28:00 chris Exp $";
+static const char rcsid[] = "$Id: auth_perl.c,v 1.25 2003/08/25 17:51:25 chris Exp $";
 
 #include <sys/types.h>
 
@@ -218,7 +218,7 @@ stringmap auth_perl_callfn(const char *perlfn, const int nvars, ...) {
             STRLEN len2;
             stringmap_insert(s, key, item_ptr(xstrdup(SvPV(val, len2))));
         }
-        SvREFCNT_dec(hashref_out);
+/*        SvREFCNT_dec(hashref_out);*/  /* `Attempt to free unreferenced scalar' */
     }
 
     SvREFCNT_dec(hashref_in);
