@@ -7,7 +7,7 @@
  *
  */
 
-static const char rcsid[] = "$Id: logging.c,v 1.4 2002/02/25 16:16:25 chris Exp $";
+static const char rcsid[] = "$Id: logging.c,v 1.6 2003/07/14 23:31:20 chris Exp $";
 
 #include <errno.h>
 #include <stdarg.h>
@@ -51,6 +51,8 @@ static struct logfac {
 
 #define NFACIL      (sizeof(facil) / sizeof(struct logfac))
 
+static int log_fac;
+
 /* log_init:
  * Start up logging. */
 void log_init(void) {
@@ -71,6 +73,8 @@ void log_init(void) {
     openlog("tpop3d", LOG_PID | LOG_NDELAY, fac);
     if (warn == 1)
         log_print(LOG_ERR, _("log_init: log-facility `%s' unknown, using `mail'"), s);
+
+    log_fac = fac;
 }
 
 
@@ -87,9 +91,9 @@ static char *verrprintf(const char *fmt, va_list ap) {
 
     *fmtbuf = 0;
 
-    /* First, we need to substitute errors into the string.
-     * XXX this would not be safe in the presence of very long format strings
-     * in the rest of the code, but we can guarantee that won't happen.... */
+    /* First, we need to substitute errors into the string. This would not be
+     * safe in the presence of very long format strings in the rest of the
+     * code, but we can guarantee that won't happen.... */
     for (p = fmt, q = strstr(p, "%m"); q; p = q, q = strstr(p, "%m")) {
         strncat(fmtbuf, p, q - p);
         strcat(fmtbuf, e);
@@ -111,7 +115,7 @@ void log_print(int priority, const char *fmt, ...) {
     va_start(ap, fmt);
     s = verrprintf(fmt, ap);
     va_end(ap);
-    syslog(priority, "%s", s);
+    syslog(priority | log_fac, "%s", s);
     if (log_stderr) fprintf(stderr, "%s\n", s);
 }
 
