@@ -26,7 +26,7 @@
 #endif /* HAVE_CONFIG_H */
 
 #ifdef MBOX_BSD
-static const char rcsid[] = "$Id: mailspool.c,v 1.38 2002/06/08 09:35:13 chris Exp $";
+static const char rcsid[] = "$Id: mailspool.c,v 1.49 2003/08/13 14:24:29 chris Exp $";
 
 #include <errno.h>
 #include <fcntl.h>
@@ -60,8 +60,9 @@ int mailspool_save_index(mailbox m);
 int mailspool_load_index(mailbox m);
 #endif /* MBOX_BSD_SAVE_INDICES */
 
-/* file_unlock:
- * Unlock a mailspool file. Returns 1 on success or 0 on failure. */
+/* file_unlock FD FILENAME
+ * Unlock a mailspool file using the open FD and given FILENAME. Returns 1 on
+ * success or 0 on failure. */
 int file_unlock(const int fd, const char *name) {
     int r = 1;
  #ifdef WITH_FCNTL_LOCKING
@@ -79,10 +80,10 @@ int file_unlock(const int fd, const char *name) {
     return r;
 }
 
-/* file_lock:
- * Lock a mailspool file. Returns 1 on success or 0 on failure. This uses
- * whatever locking strategies the user has selected with compile-time
- * definitions. */
+/* file_lock FD FILENAME
+ * Lock a mailspool file using the open FD and given FILENAME. Returns 1 on
+ * success or 0 on failure. This uses whatever locking strategies the user has
+ * selected with compile-time definitions. */
 int file_lock(const int fd, const char *name) {
     int l_fcntl, l_flock, l_dotfile;
     l_fcntl = l_flock = l_dotfile = 0;
@@ -119,8 +120,9 @@ fail:
     return 0;
 }
 
-/* mailspool_make_indexpoint:
- * Make an indexpoint. */
+/* mailspool_make_indexpoint INDEXPOINT OFFSET LENGTH MESSAGELENGTH HASH
+ * Save in INDEXPOINT the OFFSET, header LENGTH, MESSAGELENGTH and 16-byte
+ * HASH of a message. */
 void mailspool_make_indexpoint(struct indexpoint *x, const size_t offset, const size_t length, const size_t msglength, const unsigned char *hash) {
     memset(x, 0, sizeof(struct indexpoint));
 
@@ -130,20 +132,19 @@ void mailspool_make_indexpoint(struct indexpoint *x, const size_t offset, const
     if (hash) memcpy(x->hash, hash, 16);
 }
 
-/* mailspool_new_from_file:
- * Open a file, lock it, and form an index of the messages in it. */
+/* mailspool_new_from_file FILENAME
+ * Open FILENAME, lock it, and form an index of the messages in it. */
 mailbox mailspool_new_from_file(const char *filename) {
     mailbox M, failM = NULL;
     int i;
     struct timeval tv1, tv2;
     float f;
     
-    M = xcalloc(1, sizeof *M);
-    if (!M) return NULL;
+    alloc_struct(_mailbox, M);
 
     M->delete = mailspool_delete;
     M->apply_changes = mailspool_apply_changes;
-    M->send_message = mailspool_send_message;
+    M->sendmessage = mailspool_sendmessage;
 
     /* Allocate space for the index. */
     M->index = (struct indexpoint*)xcalloc(32, sizeof(struct indexpoint));
@@ -153,7 +154,7 @@ mailbox mailspool_new_from_file(const char *filename) {
         /* If the mailspool doesn't exist, fail silently, since this may be
          * getting called from find_mailbox. */
         if (errno == ENOENT) failM = MBOX_NOENT;
-        else log_print(LOG_INFO, "mailspool_new_from_file: stat(%s): %m", filename);
+        else log_print(LOG_ERR, "mailspool_new_from_file: stat(%s): %m", filename);
         goto fail;
     } else M->name = xstrdup(filename);
     
@@ -209,7 +210,7 @@ fail:
     return failM;
 }
 
-/* mailspool_delete:
+/* mailspool_delete MAILBOX
  * Deletion specific to mailspools. */
 void mailspool_delete(mailbox m) {
     if (!m) return;
@@ -219,46 +220,58 @@ void mailspool_delete(mailbox m) {
     mailbox_delete(m);
 }
 
-/* memstr:
- * Locate needle, of length n_len, in haystack, of length h_len, returning
- * NULL if it is not found. */
-static char *memstr(const char *haystack, size_t h_len, const char *needle, size_t n_len)
-{
-    const char *p;
+/* memstr HAYSTACK HLEN NEEDLE NLEN
+ * Locate NEEDLE, of length NLEN, in HAYSTACK, of length HLEN, returning NULL
+ * if it is not found. Uses the Boyer-Moore search algorithm. Cf.
+ *  http://www-igm.univ-mlv.fr/~lecroq/string/node14.html */
+static unsigned char *memstr(const unsigned char *haystack, const size_t hlen,
+                             const unsigned char *needle, const size_t nlen) {
+    int skip[256], k;
 
-    if (n_len > h_len)
-        return NULL;
+    if (nlen == 0) return (char*)haystack;
 
-    p = (const char*)memchr(haystack, *needle, h_len - n_len + 1);
-    while (p) {
-        if (!memcmp(p, needle, n_len))
-            return (char*)p;
-        else
-            p = (const char*)memchr(p + 1, *needle, h_len - n_len - (p - haystack));
+    /* Set up the finite state machine we use. */
+    for (k = 0; k < 256; ++k) skip[k] = nlen;
+    for (k = 0; k < nlen - 1; ++k) skip[needle[k]] = nlen - k - 1;
+
+    /* Do the search. */
+    for (k = nlen - 1; k < hlen; k += skip[haystack[k]]) {
+        int i, j;
+        for (j = nlen - 1, i = k; j >= 0 && haystack[i] == needle[j]; j--) i--;
+        if (j == -1) return (unsigned char*)(haystack + i + 1);
     }
 
     return NULL;
 }
 
-/* mailspool_build_index:
- * Build an index of a mailspool. Uses mmap(2) for speed. Assumes that
- * mailspools use only '\n' to indicate EOL. Returns 0 on success or -1 on
- * failure. */
+/* getmaplength LENGTH
+ * Return a multiple of the system page size which is equal to or larger than
+ * LENGTH, to be used as a parameter to mmap(2). */
+static size_t getmaplength(const size_t len) {
+    return ((len + PAGESIZE - 1) / PAGESIZE) * PAGESIZE;
+}
+
+/* mailspool_build_index MAILBOX MEMORY
+ * Build an index of a mailspool in MAILBOX. Uses mmap(2) for speed; if MEMORY
+ * is non-NULL, it is assumed to point to a mapped region on the mailspool
+ * file. Assumes that mailspools use only '\n' to indicate EOL. Returns 0 on
+ * success or -1 on failure. */
 int mailspool_build_index(mailbox M, char *filemem) {
     char *p, *q;
-    size_t len, len2;
+    size_t filelen, mappedlen = 0;
     int first = 0;
 
     if (!M || M->fd == -1) return -1;
 
-    len = len2 = M->st.st_size;
+    filelen = M->st.st_size;
 
-    if (len < 16) return 0; /* Mailspool doesn't contain any messages. */
+    if (filelen < 16)
+        goto end;
+
+    mappedlen = getmaplength(filelen);
 
-    len += PAGESIZE - (len % PAGESIZE);
     if (!filemem) {
-        filemem = mmap(0, len, PROT_READ, MAP_PRIVATE, M->fd, 0);
-        if (filemem == MAP_FAILED) {
+        if (MAP_FAILED == (filemem = mmap(0, mappedlen, PROT_READ, MAP_PRIVATE, M->fd, 0))) {
             log_print(LOG_ERR, "mailspool_build_index(%s): mmap: %m", M->name);
             return -1;
         }
@@ -275,12 +288,10 @@ int mailspool_build_index(mailbox M, char *filemem) {
         /* Nope, never seen this one before. */
         p = filemem - 2;
 
-
-    
     /* Extract all From lines from file */
     do {
         p += 2;
-        q = (char*)memchr(p, '\n', len - (p - filemem));
+        q = (char*)memchr(p, '\n', filelen - (p - filemem));
 
         if (q) {
             size_t o, l;
@@ -291,9 +302,9 @@ int mailspool_build_index(mailbox M, char *filemem) {
             mailspool_make_indexpoint(&pt, o, l, 0, NULL);
             mailbox_add_indexpoint(M, &pt);
 
-            p = memstr(q, len2 - (q - filemem), "\n\nFrom ", 7);
+            p = memstr(q, filelen - (q - filemem), "\n\nFrom ", 7);
         } else break;
-    } while (p && p < filemem + len2);
+    } while (p && p < filemem + filelen);
 
     if (first < M->num) {
         struct indexpoint *t;
@@ -335,22 +346,31 @@ int mailspool_build_index(mailbox M, char *filemem) {
     }
 #endif /* IGNORE_CCLIENT_METADATA */
 
-    munmap(filemem, len);
+end:
+    if (filemem) munmap(filemem, mappedlen);
+
     return 0;
 }
 
-/* mailspool_send_message:
- * Front-end to write_file in util.c. */
-int mailspool_send_message(const mailbox M, int sck, const int i, int n) {
+/* mailspool_sendmessage MAILBOX CONNECTION I NLINES
+ * Front-end to connection_sendmessage in util.c. */
+int mailspool_sendmessage(const mailbox M, connection c, const int i, int n) {
     struct indexpoint *x;
 
-    if (!M) return 0;
-    if (i < 0 || i >= M->num) return 0;
+    if (!M || i < 0 || i >= M->num) {
+        /* Shouldn't happen. */
+        connection_sendresponse(c, 0, _("Unable to send that message"));
+        return -1;
+    }
+
     x = M->index + i;
-    return write_file(M->fd, sck, x->offset, x->length + 1, x->msglength, n);
+    
+    /* XXX I think that this should send msglength - 1; that is, it's sending
+     * the first \n of the `\n\nFrom '. */
+    return connection_sendmessage(c, M->fd, x->offset, x->length + 1, x->msglength, n);
 }
 
-/* mailspool_apply_changes:
+/* mailspool_apply_changes MAILBOX
  * Apply deletions to a mailspool by mapping it and copying it in blocks.
  * Returns 1 on success or 0 on failure.
  *
@@ -409,8 +429,7 @@ int mailspool_apply_changes(mailbox M) {
         return 1;
     else if (M->numdeleted == M->num) {
         /* All messages deleted, so just truncate file at the beginning of the
-         * first message.
-         */
+         * first message. */
         if (ftruncate(M->fd, M->index->offset) == -1) {
             log_print(LOG_ERR, "mailspool_apply_changes(%s): ftruncate: %m", M->name);
             return 0;
@@ -418,8 +437,7 @@ int mailspool_apply_changes(mailbox M) {
     }
 
     /* We need to do something more complicated, so map the mailspool. */
-    len = M->st.st_size;
-    len += PAGESIZE - (len % PAGESIZE);
+    len = getmaplength(M->st.st_size);
     filemem = mmap(0, len, PROT_WRITE | PROT_READ, MAP_SHARED, M->fd, 0);
     if (filemem == MAP_FAILED) {
         log_print(LOG_ERR, "mailspool_apply_changes(%s): mmap: %m", M->name);
@@ -434,7 +452,7 @@ int mailspool_apply_changes(mailbox M) {
     /* Find the first message to be deleted. */
     while (I < End && !I->deleted) ++I;
     if (I == End) {
-        if (munmap(filemem, len) == -1) log_print(LOG_ERR, "mailspool_send_message: munmap: %m");
+        if (munmap(filemem, len) == -1) log_print(LOG_ERR, "mailspool_sendmessage: munmap: %m");
         log_print(LOG_ERR, _("mailspool_apply_changes(%s): inconsistency in mailspool data"), M->name);
         return 0;
     }
@@ -464,13 +482,13 @@ int mailspool_apply_changes(mailbox M) {
     /* Truncate the very end. */
     if (ftruncate(M->fd, d - filemem) == -1) {
         log_print(LOG_ERR, "mailspool_apply_changes(%s): ftruncate: %m", M->name);
-        if (munmap(filemem, len) == -1) log_print(LOG_ERR, "mailspool_send_message: munmap: %m");
+        if (munmap(filemem, len) == -1) log_print(LOG_ERR, "mailspool_sendmessage: munmap: %m");
         return 0;
     }
     
     /* Done, unmap the file. */
     if (munmap(filemem, len) == -1) {
-        log_print(LOG_ERR, "mailspool_send_message: munmap: %m");
+        log_print(LOG_ERR, "mailspool_sendmessage: munmap: %m");
         return 0;
     }
 
@@ -504,7 +522,7 @@ int mailspool_apply_changes(mailbox M) {
  * the world among the giant redwoods, collapsing suspension bridges, and
  * 1970s software empires. */
 
-/* mailspool_find_index:
+/* mailspool_find_index MAILBOX
  * Find the name of a mailspool's index file, using the spec in the config
  * file. */
 extern stringmap config;
@@ -572,7 +590,7 @@ char index_signature[] =
 "# server. If you delete this file, it will be automatically recreated by\n"
 "# tpop3d. So don't do that.\n";
 
-/* mailbox_save_index:
+/* mailbox_save_index MAILBOX
  * Save an index of a mailspool. Returns 1 on success or 0 on failure. Uses
  * stdio, which is unfortunate but makes it a bit easier to write. The
  * mailspool must be locked when this is called. Makes some attempt to avoid
@@ -650,7 +668,7 @@ fail:
     return ret;
 }
 
-/* mailspool_load_index:
+/* mailspool_load_index MAILBOX
  * Attempts to construct a mailspool index from a saved index file, if one
  * exists and is of the correct format. We may find that we need to re-parse
  * the tail of the file; this is done by calling into the `normal'
@@ -663,8 +681,9 @@ int mailspool_load_index(mailbox m) {
     char hexdigest[33] = {0};
     char sigbuf[sizeof(index_signature)];
     char *filemem = NULL;
-    size_t len, len2;
+    size_t mappedlen;
     int num, r;
+    int index_missing = 0;
 
     if (!m || m->fd == -1) goto fail;
 
@@ -673,7 +692,10 @@ int mailspool_load_index(mailbox m) {
 
     fp = fopen(indexfile, "rt");
     if (!fp) {
-        log_print(LOG_WARNING, "mailspool_load_index(%s): %m", indexfile);
+        if (errno == ENOENT)
+            index_missing = 1; /* create it at the end */
+        else
+            log_print(LOG_WARNING, "mailspool_load_index(%s): %m", indexfile);
         goto fail;
     }
 
@@ -682,10 +704,10 @@ int mailspool_load_index(mailbox m) {
     if (fstat(fileno(fp), &st) == -1) {
         log_print(LOG_ERR, "mailspool_load_index(%s): %m", indexfile);
         goto fail;
-    } else if ((st.st_mode & 0777) != 0600 || m->st.st_uid != getuid()) {
+    } else if ((st.st_mode & 0777) != 0600 || st.st_uid != getuid()) {
         log_print(LOG_ERR, _("mailspool_load_index(%s): possible security problem: index exists, but it has the wrong owner or file permissions"), indexfile);
-        log_print(LOG_ERR, _("mailspool_load_index(%s): owner is %d, should be %d; mode 0%o, should be 0600"), indexfile,
-                            m->st.st_uid, getuid(), m->st.st_mode & 0777);
+        log_print(LOG_ERR, _("mailspool_load_index(%s): owner is %d, should be %d; mode 0%03o, should be 0600"), indexfile,
+                            (int)st.st_uid, (int)getuid(), st.st_mode & 0777);
         goto fail;
     }
 
@@ -697,13 +719,9 @@ int mailspool_load_index(mailbox m) {
 
     /* Should now get a bunch of offset/hash lines. Stuff these into the
      * mailbox object. Also mmap the real mailspool so we can check these. */
-    len = len2 = m->st.st_size;
-
-    if (len < 16) goto fail;
-
-    len += PAGESIZE - (len % PAGESIZE);
-    filemem = mmap(0, len, PROT_READ, MAP_PRIVATE, m->fd, 0);
-    if (filemem == MAP_FAILED) {
+    if (m->st.st_size < 16) goto fail;
+    mappedlen = getmaplength(m->st.st_size);
+    if (MAP_FAILED == (filemem = mmap(0, mappedlen, PROT_READ, MAP_PRIVATE, m->fd, 0))) {
         log_print(LOG_ERR, "mailspool_load_index(%s): mmap: %m", m->name);
         goto fail;
     }
@@ -755,7 +773,7 @@ fail:
     /* Whatever happens, have a go at indexing the rest of the file. */
     num = m->num;
     r = mailspool_build_index(m, filemem);
-    if (m->num > num) mailspool_save_index(m);
+    if (m->num > num || index_missing) mailspool_save_index(m);
     return r;
 }
 
