@@ -7,7 +7,7 @@
  */
 
 static const char copyright[] = "$Copyright: (c) 2001 Chris Lightfoot. All rights reserved. $";
-static const char rcsid[] = "$Id: main.c,v 1.88 2003/07/14 23:31:20 chris Exp $";
+static const char rcsid[] = "$Id: main.c,v 1.91 2003/09/07 15:41:24 chris Exp $";
 
 #ifdef HAVE_CONFIG_H
 #include "configuration.h"
@@ -49,6 +49,7 @@ stringmap config;
 extern int append_domain;           /* Do we automatically try user@domain if user alone fails to authenticate? In pop3.c. */
 extern int strip_domain;            /* Do we automatically try user if user@domain fails to authenticate? */
 extern int apop_only;               /* Quit after receiving USER. */
+extern int log_bad_pass;            /* Log failing passwords. */
 int log_stderr;                     /* Are log messages also sent to standard error? */
 int verbose;                        /* Should we be verbose about data going to/from the client? */
 
@@ -167,6 +168,8 @@ int parse_listeners(const char *stmt) {
         char *regex = NULL;
 #endif
 
+        sin.sin_family = AF_INET;
+        
         s = *ll;
         
         /* Address. */
@@ -310,6 +313,7 @@ int parse_listeners(const char *stmt) {
 skip:
         xfree(host);
         xfree(port);
+        xfree(domain);
 #ifdef USE_TLS
         xfree(cert);
         xfree(pkey);
@@ -506,6 +510,12 @@ retry_pid_file:
     if (config_get_bool("apop-only"))
         apop_only = 1;
 
+    /* Should we log failing passwords? */
+    if (config_get_bool("log-bad-passwords")) {
+        log_print(LOG_WARNING, _("%s: I hope you realise that use of the log-bad-passwords option is an invasion of privacy"), configfile);
+        log_bad_pass = 1;
+    }
+
     /* Find out how long we wait before timing out.... */
     switch (config_get_int("timeout-seconds", &timeout_seconds)) {
         case -1:
