@@ -5,7 +5,7 @@
  * Copyright (c) 2002 Chris Lightfoot. All rights reserved.
  * Email: chris@ex-parrot.com; WWW: http://www.ex-parrot.com/~chris/
  *
- * $Id: tls.h,v 1.1 2002/11/10 13:30:27 chris Exp $
+ * $Id: tls.h,v 1.2 2003/09/07 15:41:24 chris Exp $
  *
  */
 
@@ -15,6 +15,6 @@
 /* tls.c */
 int tls_init(void);
 SSL_CTX *tls_create_context(const char *certfile, const char *pkeyfile);
-void tls_close(void);
+void tls_close(SSL_CTX *ctx);
 
 #endif /* __TLS_H_ */
