@@ -6,7 +6,7 @@
  *
  */
 
-static const char rcsid[] = "$Id: util.c,v 1.15 2002/06/08 09:35:13 chris Exp $";
+static const char rcsid[] = "$Id: util.c,v 1.23 2003/08/12 19:50:10 chris Exp $";
 
 #ifdef HAVE_CONFIG_H
 #include "configuration.h"
@@ -16,6 +16,7 @@ static const char rcsid[] = "$Id: util.c,v 1.15 2002/06/08 09:35:13 chris Exp $"
 #include <fcntl.h>
 #include <pwd.h>
 #include <grp.h>
+#include <signal.h>
 #include <stdarg.h>
 #include <stdio.h>
 #include <stdlib.h>
@@ -27,18 +28,19 @@ static const char rcsid[] = "$Id: util.c,v 1.15 2002/06/08 09:35:13 chris Exp $"
 #include "md5.h"
 #include "util.h"
 
-/* xwrite:
+/* xwrite FD DATA COUNT
  * Write some data, taking account of short writes and signals. */
 ssize_t xwrite(int fd, const void *buf, size_t count) {
     size_t c = count;
     const char *b = (const char*)buf;
     while (c > 0) {
-        int e = write(fd, b, c);
+        int e;
+        e = write(fd, b, c);
         if (e >= 0) {
             c -= e;
             b += e;
         } else if (errno != EINTR) return e;
-    } while (c > 0);
+    }
     return count;
 }
 
@@ -84,100 +86,6 @@ int inet_aton(const char *s, struct in_addr *ip) {
 }                                                                               
 #endif /* !HAVE_INET_ATON */
 
-/* write_file:
- * Send to socket sck the header and up to n lines of the body of a message
- * which begins at offset msgoffset + skip in the file referenced by fd, which
- * is assumed to be a mappable object. Lines which begin . are escaped as
- * required by RFC1939, and each line is terminated with `\r\n'. If n is -1,
- * the whole message is sent. Returns the number of bytes written on success
- * or 0 on failure.
- *
- * XXX Assumes that the message on disk uses only '\n' to indicate EOL.
- */
-int write_file(int fd, int sck, size_t msgoffset, size_t skip, size_t msglength, int n) {
-    char *filemem;
-    char *p, *q, *r;
-    size_t length, offset;
-    ssize_t nwritten = 0;
-    
-    offset = msgoffset - (msgoffset % PAGESIZE);
-    length = (msgoffset + msglength + PAGESIZE) ;
-    length -= length % PAGESIZE;
-
-    filemem = mmap(0, length, PROT_READ, MAP_PRIVATE, fd, offset);
-    if (filemem == MAP_FAILED) {
-        log_print(LOG_ERR, "write_file: mmap: %m");
-        return -1;
-    }
-
-    /* Find the beginning of the message headers */
-    p = filemem + (msgoffset % PAGESIZE);
-    r = p + msglength;
-    p += skip;
-
-    /* Send the message headers */
-    do {
-        q = memchr(p, '\n', r - p);
-        if (!q) q = r;
-        errno = 0;
-        
-        /* Escape a leading ., if present. */
-        if (*p == '.' && !try_write(sck, ".", 1))
-            goto write_failure;
-        ++nwritten;
-        
-        /* Send line itself. */
-        if (!try_write(sck, p, q - p) || !try_write(sck, "\r\n", 2))
-            goto write_failure;
-        nwritten += q - p + 2;
-
-        p = q + 1;
-    } while (p < r && *p != '\n');
-
-    ++p;
-
-    errno = 0;
-    if (!try_write(sck, "\r\n", 2)) {
-        log_print(LOG_ERR, "write_file: write: %m");
-        munmap(filemem, length);
-        return -1;
-    }
-    
-    /* Now send the message itself */
-    while (p < r && n) {
-        if (n > 0) --n;
-
-        q = memchr(p, '\n', r - p);
-        if (!q) q = r;
-        errno = 0;
-
-        /* Escape a leading ., if present. */
-        if (*p == '.' && !try_write(sck, ".", 1))
-            goto write_failure;
-        ++nwritten;
-        
-        /* Send line itself. */
-        if (!try_write(sck, p, q - p) || !try_write(sck, "\r\n", 2))
-            goto write_failure;
-        nwritten += q - p + 2;
-
-        p = q + 1;
-    }
-    if (munmap(filemem, length) == -1)
-        log_print(LOG_ERR, "write_file: munmap: %m");
-    
-    errno = 0;
-    if (!try_write(sck, ".\r\n", 3)) {
-        log_print(LOG_ERR, "write_file: write: %m");
-        return -1;
-    } else return nwritten + 3;
-
-write_failure:
-    log_print(LOG_ERR, "write_file: write: %m");
-    munmap(filemem, length);
-    return -1;
-}
-
 /* parse_uid:
  * Get a user id from a user name or number. Sets u and returns 1 on success,
  * or returns 0 on failure. */
@@ -201,6 +109,19 @@ int parse_uid(const char *user, uid_t *u) {
     return 0;
 }
 
+/* xsignal NUMBER HANDLER
+ * Set a signal with a similar interface to signal(2) using sigaction(2). */
+void (*xsignal(int signum, void(*handler)(int)))(int) {
+    struct sigaction sa = {0}, sa_old;
+    sigemptyset(&sa.sa_mask);
+    sa.sa_handler = handler;
+    sa.sa_flags = SA_RESTART;
+    if (sigaction(signum, &sa, &sa_old) == -1)
+        return SIG_ERR;
+    else
+        return sa_old.sa_handler;
+}
+
 /* parse_gid:
  * Get a group id from a group name or number. Sets g and returns 1 on
  * success, or returns 0 on failure. */
@@ -259,7 +180,7 @@ int unhex_digest(const char *from, unsigned char *to) {
 }
 
 #ifndef MTRACE_DEBUGGING
-/* xmalloc:
+/* xmalloc COUNT
  * Malloc, and abort if malloc fails. */
 void *xmalloc(size_t n) {
     void *v;
@@ -268,7 +189,7 @@ void *xmalloc(size_t n) {
     return v;
 }
 
-/* xcalloc:
+/* xcalloc NITEMS COUNT
  * As above. */
 void *xcalloc(size_t n, size_t m) {
     void *v;
@@ -277,7 +198,7 @@ void *xcalloc(size_t n, size_t m) {
     return v;
 }
 
-/* xrealloc:
+/* xrealloc PTR COUNT
  * As above. */
 void *xrealloc(void *w, size_t n) {
     void *v;
@@ -286,7 +207,7 @@ void *xrealloc(void *w, size_t n) {
     return v;
 }
 
-/* xfree:
+/* xfree PTR
  * Free, ignoring a passed NULL value. */
 void xfree(void *v) {
     if (v) free(v);
@@ -302,18 +223,30 @@ char *xstrdup(const char *s) {
 }
 #endif  /* !MTRACE_DEBUGGING */
 
-/* md5_digest:
- * Make an MD5 digest of some data. */
+/* xstrndup STRING COUNT
+ * Allocate a new buffer and copy in to it the first COUNT bytes of STRING,
+ * terminating it with a null. */
+char *xstrndup(const char *s, const size_t count) {
+    char *S;
+    S = xmalloc(count + 1);
+    memcpy(S, s, count);
+    S[count] = 0;
+    return S;
+}
+
+/* md5_digest DATA COUNT MD5
+ * Save in MD5 the MD5 digest of the first COUNT bytes of DATA. */
 void md5_digest(const void *v, const size_t n, unsigned char *md5) {
-    MD5_CTX ctx;
+    md5_ctx ctx;
     MD5Init(&ctx);
     MD5Update(&ctx, (unsigned char*)v, n);
     MD5Final(md5, &ctx);
 }
 
-/* md5_digest_str:
- * Make an MD5 digest in a representable format, by default hex, otherwise in
- * base64. */
+/* md5_digest_str DATA COUNT BASE64
+ * Return a static string containing a printable representation of the MD5 hash
+ * of the first COUNT bytes of DATA; in hex by default or in base64 if BASE64
+ * is nonzero. */
 char *md5_digest_str(const void *v, const size_t n, const int base64) {
     unsigned char md5[16], *p;
     static char res[33] = {0};
